/*
   InformxixTcl.c --

   This module contains the main procedures for the executables
   informixtclsh and informixwish, the entry procedure for the
   shared library Informixtcl and the procedures that create the
   TCL-Commands

   This TCL-extension is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Written by Joachim Siebold <joachim.siebold@cityweb.de>
   based on DBD::Informix (Copyright J. Leffler and others)
 
   $Header: /home/cvs/informixtcl/InformixTcl.c,v 1.4 2000/04/04 16:22:35 achim Exp $
   $Log: InformixTcl.c,v $
   Revision 1.4  2000/04/04 16:22:35  achim
   Changed package name to informixTcl, introduced namespace informixTcl

   Revision 1.3  2000/03/18 18:21:43  achim
   Modified CVS Header

*/         

#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include "InformixTcl.h"

/* ----------------- Exported procedure headings ---------------- */
    EXTERN int InformixTcl_Init _ANSI_ARGS_((Tcl_Interp *interp));



/* ----------------- Exported procedures ------------------------- */

int Informixtcl_Init (interp)
    Tcl_Interp  *interp;        /* Current interpreter. */
{
    char versionString[50];

  /*
   * Initialize Stubs mechanism, we need at least 8.1
   */
#ifdef USE_TCL_STUBS
   if (Tcl_InitStubs(interp,"8.1",0) == NULL ) {
      return TCL_ERROR;
   }
#endif         

    /*
        Initialize Informix TCL-Interface
    */

    if (InformixTclSqlInit (interp) != TCL_OK) {
        return TCL_ERROR;
    }

    sprintf (versionString, "%.2f", INFORMIX_TCL_VERSION);
    if (Tcl_PkgProvide (interp, "Informixtcl", versionString) == TCL_ERROR) {
        return TCL_ERROR;
    }

    return TCL_OK;
}

int
Informixtcl_SafeInit (interp)
    Tcl_Interp      *interp;        /* Current interpreter. */
{
    if (Informixtcl_Init (interp) == TCL_ERROR) {
        return TCL_ERROR;
    }

    Tcl_Eval (interp, "puts {Initialization succeeded}");
    return TCL_OK;
}


/*
 *----------------------------------------------------------------------
 *
 * InformixTclGetSysinfo --
 *
 *      Returns a pointer to the main global data area
 *
 * Results:
 *      Returns pointer or NULL on error
 *
 * Side effects:
 *
 *----------------------------------------------------------------------
 */
InformixSysInfo *
InformixTclGetSysinfo ()
{
    static InformixSysInfo *ptr= (InformixSysInfo *) NULL;

    if(ptr == (InformixSysInfo *) NULL ) {
        ptr= (InformixSysInfo *) ckalloc(sizeof(InformixSysInfo));
    }
    return(ptr);
}


/*
 *----------------------------------------------------------------------
 *
 * InformixTclSqlInit --
 *
 *      Initializes this module by registering all the command procedures.
 *
 * Results:
 *      The return value is normally TCL_OK. If there are problems by
 *      evaluating the startup script informixsql.tcl, TCL_ERROR will be 
 *      returned.
 *
 * Side effects:
 *      The tcl commands of the InformixTcl extension are defined in the given
 *      interpreter.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclSqlInit (interp)
    Tcl_Interp      *interp;        /* Current interpreter. */
{
    ObjProcTable    *pObj;          /* index pointer in proc table. */
    InformixSysInfo *ptr;           /* Pointer to global data */
    char            result[100];    /* Program Version string */
    char            varname[100];   /* Varname for TCL_LinkVar */
    Tcl_DString     buf;            /* Script Buffer */

    char            *script=
"namespace eval ::informixTcl {
namespace export inflogon
namespace export inflogoff
namespace export infcommit
namespace export infrollback
namespace export inffetch
namespace export infprepare
namespace export infexecute
namespace export infclose
namespace export infrelease
namespace export infget
namespace export infset
namespace export inflogoffall
namespace export infdosql
}";

    ObjProcTable informixObjProcs[] = {
        { "::informixTcl::inflogon",  (Tcl_ObjCmdProc *) InformixTclLogonCmd },
        { "::informixTcl::inflogoff", (Tcl_ObjCmdProc *) InformixTclLogoffCmd },
        { "::informixTcl::infcommit", (Tcl_ObjCmdProc *) InformixTclCommitCmd },
        { "::informixTcl::infrollback",
                     (Tcl_ObjCmdProc *) InformixTclRollbackCmd },
        { "::informixTcl::inffetch",   (Tcl_ObjCmdProc *) InformixTclFetchCmd },
        { "::informixTcl::infprepare",
                     (Tcl_ObjCmdProc *) InformixTclPrepareCmd },
        { "::informixTcl::infexecute", 
                     (Tcl_ObjCmdProc *) InformixTclExecuteCmd },
        { "::informixTcl::infclose",   (Tcl_ObjCmdProc *) InformixTclCloseCmd },
        { "::informixTcl::infrelease", 
                     (Tcl_ObjCmdProc *) InformixTclReleaseCmd},
        { "::informixTcl::infget",  (Tcl_ObjCmdProc *) InformixTclGetInfoCmd },
        { "::informixTcl::inflogoffall",
                     (Tcl_ObjCmdProc *) InformixTclDisconnectAllCmd },
        { "::informixTcl::infset",   (Tcl_ObjCmdProc *) InformixTclSetCmd },
        { "::informixTcl::infdosql",  (Tcl_ObjCmdProc *) InformixTclDosqlCmd },
        { (char *) NULL,    (Tcl_ObjCmdProc *) NULL }
    };

    /*
        Define all the tcl commands, that are mentioned in proc table.
    */

    for (pObj = informixObjProcs; pObj->name; pObj++) {
        Tcl_CreateObjCommand (interp, pObj->name, pObj->proc,
            (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
    }

    /*
        Allocate System Global Date area
    */
    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *)  NULL) {
        AppendResult (interp, 
            "Cannot initialize Informix System Global Data\"");
        return TCL_ERROR;
    }

    /*
        Initialize Hash Tables
    */

    Tcl_InitHashTable(&ptr->statementHash, TCL_STRING_KEYS);
    Tcl_InitHashTable(&ptr->logonHash, TCL_STRING_KEYS);

    /*
        Initialize other global TCL-Interface data
     */
    ptr->debug= 0;
#if ESQLC_VERSION >= 600
    ptr->multiconnections = 1;       /* Multiple connections allowed */
#else
    ptr->multiconnections = 0;       /* Multiple connections forbidden */
#endif /* ESQLC_VERSION */     

    /*
        Initialize Message Array
    */
    sprintf(result,"informixTcl Version %1.1f",INFORMIX_TCL_VERSION);
    InformixSetMsgString(interp, InformixVersion, result);
    InformixSetMsgString(interp, InformixErrortxt,"");
    InformixSetMsgInt(interp, InformixErrornum,(long) 0);
    InformixSetMsgInt(interp, InformixProdVersion,(long) ESQLC_VERSION);

    strcpy(varname,"::informixTcl::infmsg(debug)");
    Tcl_LinkVar(interp,varname,(char *) &ptr->debug,TCL_LINK_INT);
    strcpy(varname,"::informixTcl::infmsg(multiconnections)");
    Tcl_LinkVar(interp,varname,(char *) &ptr->multiconnections,
        TCL_LINK_BOOLEAN | TCL_LINK_READ_ONLY);

    /*
        No current connection
    */
    ptr->current_connection= (LogonInfo *) NULL;

    /*
        Create Exit Handler
    */
    Tcl_CreateExitHandler((Tcl_ExitProc *) InformixApiDisconnectAll, 
        (ClientData) interp);

    /*
        Run script to export procs and global variable
    */
    Tcl_DStringInit(&buf);
    Tcl_DStringAppend(&buf,script,-1);
    if(Tcl_Eval(interp,Tcl_DStringValue(&buf))!= TCL_OK) {
        Tcl_DStringFree(&buf);
        return(TCL_ERROR);
    }
    Tcl_DStringFree(&buf);

    return TCL_OK;
}
/*
 *----------------------------------------------------------------------
 *
 * InformitTclPrepareCmd --
 *
 *      Implements the tcl command infprepare with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK; in this case interp->results
 *      will be set to the name of the fresh created StatementHandle. If there
 *      are malformed parameters or any problems while createing the cursor,
 *      TCL_ERROR will be returned.
 *
 * Side effects:
 *      The hashtable of StatementHandle will have an additional line, which
 *      contains a structure with information about the newly created cursor.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclPrepareCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *logonName;     /* Name of logonHandle. */
    LogonInfo       *logonVars;     /* logonHandle identified by 1. param. */
    StatementInfo   *statementVars; /* the newly created StatementHandle. */
    char            *cmd;           /* SQL-Statement (2nd Parameter */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */

    /*
        Check, wheter there are at least 2 parameters, the logon-handle
        and the sql statement
    */

    if (objc != 3) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj (objv[0], &dummyLength));
        AppendResult (interp, " Logon-Handle SQL-Statement\"");
        return TCL_ERROR;
    }

    /*
        Get Logon Handle
    */
    logonName = Tcl_GetStringFromObj (objv[1], &dummyLength);
    if (InformixTclLogonHandle (interp, logonName, &logonVars) == TCL_ERROR) {
        return TCL_ERROR;
    }

    /*
        Get SQL Statement 
    */
    cmd = Tcl_GetStringFromObj (objv[2], &dummyLength);

    /*
        Create statement handle
    */
    if(InformixTclNewStatementHandle(interp,logonVars,&statementVars)!=TCL_OK) {
        return(TCL_ERROR);
    }

    /*
        Call the procedure, which prepares the statement and 
        allocates some information about it.
    */

    if (InformixApiPrepare (interp, statementVars,cmd)==TCL_ERROR) {
        InformixTclDeleteStatementHandle(interp,statementVars);
        return TCL_ERROR;
    }

    Tcl_ResetResult (interp);
    AppendResult (interp, statementVars->statementName);

    /*
        Set statement valid
    */
    statementVars->magic= STATEMENT_MAGIC;
    return (TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclCloseCmd --
 *
 *      Implements the tcl command infclose with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid cursorName
 *      given, TCL_ERROR will be returned; in this case interp->results will
 *      be set to the error message.
 *
 * Side effects:
 *      The hashtable of StatementHandle will have one fewer line, which used to
 *      contain a structure with information about the destroyed cursor.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclCloseCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *statementName; /* Name of statement handle. */
    StatementInfo   *statementVars; /* the StatementHandle to destroy. */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */
    int             erg;            /* return value */

    /*
        Check, if the one and only parameter is a valid StatementHandle.
    */

    if (objc != 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj (objv[0], &dummyLength));
        AppendResult (interp, " StatementHandle\"");
        return TCL_ERROR;
    }
    statementName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclStatementHandle (interp, statementName,
            &statementVars)!=TCL_OK) {
        return TCL_ERROR;
    }

    /*
        Call the procedure which closes the statement 
    */
    erg= InformixApiClose (interp, statementVars); 

    return(erg);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclReleaseCmd --
 *
 *      Implements the tcl command infrelease with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid cursorName
 *      given, TCL_ERROR will be returned; in this case interp->results will
 *      be set to the error message.
 *
 * Side effects:
 *      The hashtable of StatementHandle will have one fewer line, which used to
 *      contain a structure with information about the destroyed cursor.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclReleaseCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *statementName; /* Name of statement handle. */
    StatementInfo   *statementVars; /* the StatementHandle to destroy. */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */
    int             erg;            /* return value */

    /*
        Check, if the one and only parameter is a valid StatementHandle.
    */

    if (objc != 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj (objv[0], &dummyLength));
        AppendResult (interp, " StatementHandle\"");
        return TCL_ERROR;
    }
    statementName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclStatementHandle (interp, statementName,
            &statementVars)!=TCL_OK) {
        return TCL_ERROR;
    }

    /*
        Delete Statement
    */
    erg= InformixApiRelease(interp,statementVars);
    if (erg!= TCL_OK) return(TCL_ERROR);

    /*
        Invalidate and delete statement 
    */
    statementVars->magic= 0;

    if(InformixTclDeleteStatementHandle(interp,statementVars)!= TCL_OK) {
        return(TCL_ERROR);
    }
    return(erg);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclExecuteCmd --
 *
 *      Implements the tcl command infexecute with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid cursorName
 *      or any malformed combination of options given, TCL_ERROR will be
 *      returned; in this case interp->results will be set to the error message.
 *      The same goes, if the database kernel returned a returncode <> 0.
 *
 * Side effects:
 * none
 *
 *----------------------------------------------------------------------
 */

int
InformixTclExecuteCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *statementName; /* Name of statement Handle */
    StatementInfo   *statementVars; /* Statement Handle */
    int             dummyLength;
    int             rc;

    /*
        Check, if there are at least the Statement Handle and a SQL statement.
    */

    if (objc < 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " statementHandle ?Parameters?\"");
        return TCL_ERROR;
    }

    statementName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclStatementHandle (interp, statementName, &statementVars) 
        == TCL_ERROR) {
        return TCL_ERROR;
    }

    /*
        Call the procedure , which handles the SQL request.
    */

    rc=InformixApiExecute (interp, statementVars, objc-2 , objv+2);
    return(rc);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclFetchCmd --
 *
 *      Implements the tcl command inffetch with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid cursorName
 *      or any malformed combination of options given, TCL_ERROR will be
 *      returned; in this case interp->results will be set to the error message.
 *
 *      If the returnvalue is TCL_OK, interp->result will point to a valid
 *      tcl list, that is the represantation of the fetched row. 
 *
 * Side effects:
 *      The cursor is moved, so that the next positional fetch will return a
 *      different row.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclFetchCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *statementName; /* Name of statement handle. */
    StatementInfo   *statementVars; /* the StatementHandle to destroy. */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */

    /*
        Check, if the one and only parameter is a valid StatementHandle.
    */

    if (objc < 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj (objv[0], &dummyLength));
        AppendResult (interp, " StatementHandle ?var1? ?var2? ?var...?\"");
        return TCL_ERROR;
    }
    statementName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclStatementHandle (interp, statementName,
            &statementVars)!=TCL_OK) {
        return TCL_ERROR;
    }

    /*
        Call the procedure which closes the statement 
    */


    return InformixApiFetch (interp, statementVars, objc-2, objv+2);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixTclCommitCmd --
 *
 *      Implements the tcl command infcommit with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid logonName
 *      given, TCL_ERROR will be returned; in this case interp->results will
 *      be set to the error message.
 *
 * Side effects:
 *      All SQL statements of the current logon are commited.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclCommitCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *logonName;     /* Name of logonHandle. */
    LogonInfo       *logonVars;     /* logonHandle identified by 1. param. */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */

    /*
        Check, if the only parameter is the name of a valid logonHandle.
    */

    if (objc != 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " Logon-Handle\"");
        return TCL_ERROR;
    }
    logonName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclLogonHandle (interp, logonName, &logonVars) == TCL_ERROR) {
        return TCL_ERROR;
    }

    /*
        Call the procedure, which sends a COMMIT request
        to the database kernel.
    */

    return InformixApiCommit (interp, logonVars);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclRollbackCmd --
 *
 *      Implements the tcl command infrollback with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid logonName
 *      given, TCL_ERROR will be returned; in this case interp->results will
 *      be set to the error message.
 *
 * Side effects:
 *      All SQL statements of the current logon are rollbacked.
 *
 *----------------------------------------------------------------------
 */

int 
InformixTclRollbackCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
char            *logonName;         /* Name of logonHandle. */
LogonInfo       *logonVars;         /* logonHandle identified by 1. param. */
int             dummyLength;        /* Out parameter of GetXxxFromObj. */

    /*
        Check, if the only parameter is the name of a valid logonHandle.
    */

    if (objc != 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj (objv[0], &dummyLength));
        AppendResult (interp, " Logon-Handle\"");
        return TCL_ERROR;
    }
    logonName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclLogonHandle (interp, logonName, &logonVars) == TCL_ERROR) {
        return TCL_ERROR;
    }

    /*
        Call the procedure , which sends a ROLLBACK request
        to the database kernel.
    */

    return InformixApiRollback (interp, logonVars);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixTclLogonCmd --
 *
 *      Implements the tcl command inflogon with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there are invalid options or
 *      the attempt to connect to the database fails for any reason,
 *      TCL_ERROR will be returned; in this case interp->results will
 *      be set to the error message.
 *
 * Side effects:
 *      There is a new logonHandle with an open database connection accessible
 *      under the returned name.
 *
 *----------------------------------------------------------------------
 */

int
InformixTclLogonCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument Objects. */
{
    int      dummyLength;       /* Out parameter of GetXxxFromObj. */
    LogonInfo *logonVars;       /* Logon Info */
    char    *database;          /* Database Name */
    char    *username;          /* User Name */
    char    *password;          /* Password */
    char    *option;            /* Connection Option */
    int     conn_opt;           /* Connection Option */

    /*
        Parse and get Parameters
    */
    database = (char *) NULL;
    username = (char *) NULL;
    password = (char *) NULL;
    if (objc <1 || objc >5) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " ?Databasename? ?Username? ?Password? ?Option?\"");
        return TCL_ERROR;
    }
    if(objc > 1) database = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if(objc > 2) username = Tcl_GetStringFromObj ( objv[2], &dummyLength);
    if(objc > 3) password = Tcl_GetStringFromObj ( objv[3], &dummyLength);

    /*
        Connection Option is optional. Only "AutoCommit" is supported"
    */
    if(objc==5) {
        option= Tcl_GetStringFromObj (objv[4], &dummyLength);
        if(strcmp(option,"AutoCommit")==0) {
            conn_opt= INFORMIX_CONNOPT_AUTOCOMMIT;
        } else {
            AppendResult (interp, " Connection option must be \"AutoCommit\"");
            return TCL_ERROR; 
        }
    } else {
        conn_opt= INFORMIX_CONNOPT_NONE;
    }

    /*
        Create new Logon Handle 
    */
    if(InformixTclNewLogonHandle(interp,&logonVars)!=TCL_OK) {
        return(TCL_ERROR);
    }

    /*
        Now open the connection to Informix with the given parameter.
    */

    if ( InformixApiLogon (interp, logonVars, database, username, password,
        conn_opt)== TCL_ERROR) {
        InformixTclDeleteLogonHandle(interp,logonVars);
        return TCL_ERROR;
    }

    /*
        Set Logon Handle valid and return logon handle
    */
    logonVars->magic= LOGON_MAGIC;

    Tcl_ResetResult (interp);
    AppendResult (interp, logonVars->logonName);
    return TCL_OK;

}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclLogoffCmd --
 *
 *      Implements the tcl command inflogoff with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid logonName
 *      given, TCL_ERROR will be returned; in this case interp->results will
 *      be set to the error message.
 *
 * Side effects:
 *      None.
 *
 *----------------------------------------------------------------------
 */

int 
InformixTclLogoffCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument Objects. */
{
    char            *logonName;     /* Name of logonHandle. */
    LogonInfo       *logonVars;     /* logonHandle identified by 1. param. */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */
    int             erg;            /* Return status */

    /*
        Check, if the only parameter is the name of a valid logonHandle.
    */

    if (objc != 2) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " Logon-Handle\"");
        return TCL_ERROR;
    }
    logonName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclLogonHandle (interp, logonName, &logonVars) == TCL_ERROR) {
        return TCL_ERROR;
    }
  
    /*
        Invalidate Logon Handle
    */
    logonVars->magic=0;

    /*
        Call the procedure, which closes the connection.
    */

    erg= TCL_OK;
    erg=InformixApiLogoff (interp, logonVars);

    /*
        Delete LogonHandle
    */
    if(InformixTclDeleteLogonHandle(interp,logonVars)!= TCL_OK) {
        return(TCL_ERROR);
    }
    return(erg);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclStatementHandle --
 *
 *      Looks up the given statementName in the statement hash table.
 *
 * Results:
 *      The return value is normally a pointer to the StatementInfo structure
 *      of the found hash entry. If no Statement with the given name exists,
 *      NULL is returned.
 *
 * Side effects:
 *      none
 *
 *----------------------------------------------------------------------
 */
int
InformixTclStatementHandle (interp, statementName, statementVars)
    Tcl_Interp      *interp;            /* Current interpreter. */
    char            *statementName;     /* Name to look for. */
    StatementInfo   **statementVars;    /* Found StatementHandle to return. */
{
    Tcl_HashEntry   *hashEntry;         /* Entry with statement name 
                                            and infoPrt. */
    InformixSysInfo *ptr;               /* Pointer to system global area */

    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *) NULL) return(TCL_ERROR);

    if (!(hashEntry = Tcl_FindHashEntry (&ptr->statementHash, statementName))) {
        AppendResult (interp, "No valid statement handle: ");
        AppendResult (interp, statementName);
        return (TCL_ERROR);
    }
    *statementVars = (StatementInfo *) Tcl_GetHashValue (hashEntry);
    if (InformixTclCheckStatement (interp, *statementVars) == TCL_ERROR) {
        return (TCL_ERROR);
    }

    return (TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclNewStatementHandle --
 *
 *      Creates Data Structures for a new statement handle and creates
 *      hash entry
 *
 * Results:
 *      The return value is normally a pointer to the StatementInfo structure
 *      of the created hash entry. If the creation of the new handle fails,
 *      NULL is returned.
 *
 *      On success only the following data fields of the statement handle
 *      are initialized:
 *      - logonHandle
 *      - statementName
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclNewStatementHandle (interp,logonVars,statementVars)
    Tcl_Interp          *interp;        /* Current interpreter. */
    LogonInfo           *logonVars;     /* Logon handle */
    StatementInfo       **statementVars;/* New StatementHandle to return. */
{
    Tcl_HashEntry       *newEntry;      /* Entry with statement name and 
                                            infoPrt. */
    int                 added;          /* Hash entry successfully added? */
    int                 cnt;            /* counter */
    InformixSysInfo     *ptr;           /* pointer to system global area */


    Tcl_ResetResult (interp);
    if (InformixTclCheckLogon (interp, logonVars) == TCL_ERROR) {
        AppendResult (interp, 
            "New statement handle not allocated -- invalid logon handle");
        return (TCL_ERROR);
    }

    if ((*statementVars = (StatementInfo *) ckalloc (sizeof (StatementInfo))) 
        == NULL) {
        AppendResult (interp, 
            "Allocation of statement handle failed due to lack of memory");
        return (TCL_ERROR);
    }
    (*statementVars)->logonVars    = logonVars;

    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *) NULL) return(TCL_ERROR);
    /*
        Store the new StatementHandle into our hashtable with its name as key
        Put the name of the statement into the StatementInfo.
    */

    added = 0;
    cnt   = 0;

    while (!added) {
        sprintf ((*statementVars)->statementName, "ifxstmt%d", ++cnt);
        newEntry = Tcl_CreateHashEntry (&ptr->statementHash, 
                    (*statementVars)->statementName, &added);
    }
    Tcl_SetHashValue (newEntry, *statementVars);

    return (TCL_OK);
}


/*
 *----------------------------------------------------------------------
 *
 * InformixTclDeleteStatementHandle --
 *
 *  Deletes the entry corresponding to a statement from the hash table and
 *  frees memory of the statement handle
 *
 * Results:
 *      If the deletion of the handle fails, NULL is returned.
 *
 *      Warning:
 *      The procedure does not free memory that is referenced by
 *      pointers within the statement handle
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclDeleteStatementHandle (interp,statementVars)
    Tcl_Interp          *interp;            /* Current interpreter. */
    StatementInfo       *statementVars;     /* Statement to be freed. */
{
    Tcl_HashEntry       *oldEntry;          /* hash table entry of 
                                                statementName. */
    InformixSysInfo     *ptr;               /* pointer to system global area */


    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *) NULL) return(TCL_ERROR);

    if (!(oldEntry = Tcl_FindHashEntry (&ptr->statementHash, 
                   statementVars->statementName))) {
        SetResult (interp, "Oops, statement not found in hash table???");
        return TCL_ERROR;
    }
    statementVars->magic = 0;
    ckfree ((char *) statementVars);

    /*
        Delete the StatementHandle from our hashtable with its name as key.
    */

    Tcl_DeleteHashEntry (oldEntry);
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclCheckStatement --
 *
 *      Checks whether the given statement handle is known and alive.
 *
 * Results:
 *      The return value is normally TCL_OK; if the statement is invalid,
 *      TCL_ERRORNULL is returned and interp->result will point to the
 *      error message.
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclCheckStatement (interp, statement)
    Tcl_Interp      *interp;        /* Current interpreter. */
    StatementInfo   *statement;     /* statement handle. */
{
    if (!statement) {
        SetResult (interp, "Invalid StatementHandle (NULL)");
        return TCL_ERROR;
    }
    if (statement->magic != STATEMENT_MAGIC) {
        SetResult (interp, "Invalid StatementHandle (already dead)");
        return TCL_ERROR;
    }
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclLogonHandle --
 *
 *      Looks up the given logonName in the logon hash table.
 *
 * Results:
 *      The return value is normally a pointer to the LogonInfo structure
 *      of the found hash entry. If no logon with the given name exists,
 *      NULL is returned.
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclLogonHandle (interp, logonName, logonVars)
    Tcl_Interp      *interp;        /* Current interpreter. */
    char            *logonName;     /* Name to look for. */
    LogonInfo       **logonVars;    /* Found logonHandle to return. */
{
Tcl_HashEntry       *hashEntry;     /* Entry with logon name and infoPrt. */
InformixSysInfo     *ptr;           /* pointer to system global area */

    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *) NULL) return(TCL_ERROR);
    if (!(hashEntry = Tcl_FindHashEntry (&ptr->logonHash, logonName))) {
        AppendResult (interp, "No valid logon handle: ");
        AppendResult (interp, logonName);
        return (TCL_ERROR);
    }
    *logonVars = (LogonInfo *) Tcl_GetHashValue (hashEntry);
    if (InformixTclCheckLogon (interp, *logonVars) == TCL_ERROR) {
        return (TCL_ERROR);
    }

    return (TCL_OK);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclNewLogonHandle --
 *
 *      Creates Data Structures for a new logon handle and creates
 *      hash entry
 *
 * Results:
 *      The return value is normally a pointer to the LogonInfo structure
 *      of the created hash entry. If the creation of the new handle fails,
 *      NULL is returned.
 *
 *      On success only the following data fields of the logon handle
 *      are initialized:
 *      - magic
 *      - logonName
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclNewLogonHandle (interp, logonVars)
    Tcl_Interp      *interp;        /* Current interpreter. */
    LogonInfo       **logonVars;    /* New logonHandle to return. */
{
    Tcl_HashEntry   *newEntry;      /* Entry with logon name and infoPrt. */
    int             added;          /* Hash entry successfully added? */
    int             cnt;            /* counter */
    InformixSysInfo *ptr;           /* pointer to system global area */

    Tcl_ResetResult (interp);

    if ((*logonVars = (LogonInfo *) ckalloc (sizeof (LogonInfo))) == NULL) {
        AppendResult (interp, 
            "Allocation of Logon-Handle failed due to lack of memory");
        return (TCL_ERROR);
    }

    /*
        Store the new logonHandle into our hashtable with its name as key
        Put the name of the logon into the StatementInfo.
    */

    added = 0;
    cnt   = 0;

    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *) NULL) return(TCL_ERROR);

    while (!added) {
        sprintf ((*logonVars)->logonName, "ifxconn%d", ++cnt);
        newEntry = Tcl_CreateHashEntry (&ptr->logonHash, 
            (*logonVars)->logonName, &added);
    }
    Tcl_SetHashValue (newEntry, *logonVars);

    return (TCL_OK);
}


/*
 *----------------------------------------------------------------------
 *
 * InformixTclDeleteLogonHandle --
 *
 *  Deletes the entry corresponding to a logon from the hash table and
 *  frees memory of the logon handle
 *
 * Results:
 *      If the deletion of the handle fails, NULL is returned.
 *
 *      Warning:
 *      The procedure does not free memory that is referenced by
 *      pointers within the logon handle
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclDeleteLogonHandle (interp,logonVars)
    Tcl_Interp      *interp;            /* Current interpreter. */
    LogonInfo       *logonVars;         /* Logon to be freed. */
{
    Tcl_HashEntry   *oldEntry;          /* hash table entry of logonName. */
    InformixSysInfo *ptr;               /* pointer to system global data */


    ptr= InformixTclGetSysinfo ();
    if(ptr== (InformixSysInfo *) NULL) return(TCL_ERROR);
    if (!(oldEntry = Tcl_FindHashEntry (&ptr->logonHash, 
                                  logonVars->logonName))) {
        SetResult (interp, "Oops, logon not found in hash table???");
        return TCL_ERROR;
    }
    ckfree ((char *) logonVars);

    /*
        Delete the logonHandle from our hashtable with its name as key.
    */

    Tcl_DeleteHashEntry (oldEntry);
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclCheckLogon --
 *
 *      Checks whether the given logon is known and alive.
 *
 * Results:
 *      The return value is normally TCL_OK; if the logon is invalid,
 *      TCL_ERRORNULL is returned and interp->result will point to the
 *      error message.
 *
 * Side effects:
 *      None
 *
 *----------------------------------------------------------------------
 */

int
InformixTclCheckLogon (interp, logon)
    Tcl_Interp      *interp;        /* Current interpreter. */
    LogonInfo       *logon;         /* Logon handle. */
{
    if (!logon) {
        SetResult (interp, "Invalid Logon-Handle (NULL)");
        return TCL_ERROR;
    }
    if (logon->magic != LOGON_MAGIC) {
        SetResult (interp, "Invalid Logon-Handle (already dead)");
        return TCL_ERROR;
    }
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclGetInfoCmd --
 *
 *      Implements the tcl command infgetinfo with the tcl8.0 string interface.
 *
 * Results:
 *      If there is an invalid cursorName
 *      or statement name or info is requested not valid for the type of
 *      handle, TCL_ERROR is returned.
 *      Ohterwise the requested information is returned
 *
 *
 * Side effects:
 *
 *----------------------------------------------------------------------
 */

int
InformixTclGetInfoCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *nmHandle;      /* Name of statement handle. */
    char            *request;       /* Requested Information */
    StatementInfo   *statementVars; /* StatementHandle */
    LogonInfo       *logonVars;     /* Logon Handle */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */
    int             handle_is_logon;/* Type of handle */
    int             rc;

    rc= TCL_OK;

    if (objc != 3) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " logon_or_statementHandle inforequest\"");
        return TCL_ERROR;
    }
    nmHandle= Tcl_GetStringFromObj( objv[1], &dummyLength);
    request= Tcl_GetStringFromObj( objv[2], &dummyLength);

    /*
        Determine kind of handle
    */
    handle_is_logon=1;
    if(strncmp(nmHandle,"ifxst",5)==0) handle_is_logon= 0;

    /*
        Get Info
    */
    if(handle_is_logon) { 
        if (InformixTclLogonHandle (interp, nmHandle,
                 &logonVars) == TCL_ERROR) {
            return TCL_ERROR;
        }
    } else {
        if (InformixTclStatementHandle (interp, nmHandle, 
                &statementVars) == TCL_ERROR) {
            return TCL_ERROR;
        }
    }


    if(handle_is_logon) {
        if(strcmp(request,"ConnectionName")==0) 
            AppendResult(interp,logonVars->logonName);
        else if(strcmp(request,"DatabaseName")==0)
            AppendResult(interp,logonVars->database);
        else if(strcmp(request,"IsConnected")==0)
            AppendResult(interp,(logonVars->is_connected) ? "1":"0");
        else if(strcmp(request,"IsOnlineDb")==0)
            AppendResult(interp,(logonVars->is_onlinedb) ? "1":"0");
        else if(strcmp(request,"IsModeAnsi")==0)
            AppendResult(interp,(logonVars->is_modeansi) ? "1":"0");
        else if(strcmp(request,"IsLoggedDb")==0)
            AppendResult(interp,(logonVars->is_loggeddb) ? "1":"0");
        else if(strcmp(request,"IsInTransaction")==0)
            AppendResult(interp,(logonVars->is_txactive) ? "1":"0");
        else if(strcmp(request,"HasAutoCommit")==0)
            AppendResult(interp,(logonVars->has_autocommit) ? "1":"0");
        else if(strcmp(request,"ChopBlanks")==0)
            AppendResult(interp,(logonVars->chop_blanks) ? "1":"0");
        else {
            AppendResult(interp,"Invalid infget request, must be: ");
            AppendResult(interp,"ConnectionName | DatabaseName | IsConnected | IsOnlineDb | IsModeAnsi | IsLoggedDb | IsInTransaction | HasAutoCommit");
            return(TCL_ERROR);
        }
    } else {
        if(strcmp(request,"State")==0) {
            switch(statementVars->st_state) {
            case Unused: AppendResult(interp,"Unused");
                            break;
            case Prepared:  AppendResult(interp,"Prepared");
                            break;
            case Allocated: AppendResult(interp,"Allocated");
                            break;
            case Described: AppendResult(interp,"Described");
                            break;
            case Declared:  AppendResult(interp,"Declared");
                            break;
            case Opened:    AppendResult(interp,"Opened");
                            break;
            case Finished:  AppendResult(interp,"Finished");
                            break;
            }
        } else if(strcmp(request,"NBlobs")==0) 
            SetIntResult(interp,statementVars->n_blobs);
        else if(strcmp(request,"NCols")==0) 
            SetIntResult(interp,statementVars->n_columns);
        else if(strcmp(request,"NBound")==0) 
            SetIntResult(interp,statementVars->n_bound);
        else if(strcmp(request,"NRows")==0) 
            SetIntResult(interp,statementVars->n_rows);
        else if(strcmp(request,"NInputParams")==0) 
            SetIntResult(interp,statementVars->n_input_params);
        else if(strcmp(request,"ColNames")==0) 
            rc=InformixApiGetAttrib(interp,statementVars,0);
        else if(strcmp(request,"ColNullable")==0) 
            rc=InformixApiGetAttrib(interp,statementVars,1);
        else if(strcmp(request,"ColType")==0) 
            rc=InformixApiGetAttrib(interp,statementVars,2);
        else if(strcmp(request,"ColLength")==0) 
            rc=InformixApiGetAttrib(interp,statementVars,3);
        else if(strcmp(request,"ColScale")==0) 
            rc=InformixApiGetAttrib(interp,statementVars,4);
        else if(strcmp(request,"ColPrecision")==0) 
            rc=InformixApiGetAttrib(interp,statementVars,5);
        else {
            AppendResult(interp,"Invalid information request, must be: ");
            AppendResult(interp,"State | NBlobs | NCols | NBound | NRows | NInputParams | ColNames | ColNullable | ColType | ColLength | ColScale | ColPrecision");
            return(TCL_ERROR);
        }
    }
    return(rc);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclSetCmd
 *
 *      Implements the tcl command infset with the tcl8.0 string interface.
 *
 * Results:
 *      If there is an invalid cursorName
 *      or statement name or info is requested not valid for the type of
 *      handle, TCL_ERROR is returned.
 *      Ohterwise the requested parameter set.
 *
 *
 * Side effects:
 *
 *----------------------------------------------------------------------
 */

int
InformixTclSetCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *nmHandle;      /* Name of statement handle. */
    char            *parameter;     /* Name of Parameter */
    char            *value;         /* Value */
    StatementInfo   *statementVars; /* StatementHandle */
    LogonInfo       *logonVars;     /* Logon Handle */
    int             dummyLength;    /* Out parameter of GetXxxFromObj. */
    int             handle_is_logon;/* Type of handle */
    int             rc;

    rc= TCL_OK;

    if (objc != 4) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " logon_or_statementHandle parameter value\"");
        return TCL_ERROR;
    }
    nmHandle= Tcl_GetStringFromObj( objv[1], &dummyLength);
    parameter= Tcl_GetStringFromObj( objv[2], &dummyLength);
    value= Tcl_GetStringFromObj( objv[3], &dummyLength);

    /*
        Determine kind of handle
    */
    handle_is_logon=1;
    if(strncmp(nmHandle,"ifxst",5)==0) handle_is_logon= 0;

    /*
        Get Info
    */
    if(handle_is_logon) { 
        if (InformixTclLogonHandle (interp, nmHandle,
                 &logonVars) == TCL_ERROR) {
            return TCL_ERROR;
        }
    } else {
        if (InformixTclStatementHandle (interp, nmHandle, 
                &statementVars) == TCL_ERROR) {
            return TCL_ERROR;
        }
    }

    if(!handle_is_logon) {
    } else {
        if(strcmp(parameter,"ChopBlanks")==0) {
            if(strcmp(value,"1")==0) {
                logonVars->chop_blanks= 1;
            } else if(strcmp(value,"0")==0) {
                logonVars->chop_blanks= 0;
            } else {
                AppendResult (interp, Tcl_GetStringFromObj ( objv[0], 
                    &dummyLength));
                AppendResult (interp, 
                    ": illegal value for parameter ChopBlanks.");
                AppendResult(interp, 
                    " Must be 0 or 1");
                return(TCL_ERROR);
            }
        } else {
            AppendResult (interp, Tcl_GetStringFromObj ( objv[0], 
                &dummyLength));
            AppendResult (interp, 
                ": illegal parameter for logon handle");
            return(TCL_ERROR);
        }
    }
    return(TCL_OK);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixTclDisconnectAllCmd
 *
 *      Implements the tcl command infdisconnectall with the tcl8.0 
 *      string interface.
 *
 * Results:
 *
 * Side effects:
 *
 *----------------------------------------------------------------------
 */

int
InformixTclDisconnectAllCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    int             rc;
    int             dummyLength;

    rc= TCL_OK;

    if (objc != 1) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp,"\"");
        return TCL_ERROR;
    }
    rc= InformixApiDisconnectAll(interp);
    return(rc);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixTclDosqlCmd --
 *
 *      Implements the tcl command infdosql with the tcl8.0 string interface.
 *
 * Results:
 *      The return value is normally TCL_OK. If there is an invalid logonName
 *      TCL_ERROR will be returned; in this case interp->results will be set 
 *      to the error message. The same goes, if the database kernel returned 
 *      a returncode <> 0.
 *
 * Side effects:
 * none
 *
 *----------------------------------------------------------------------
 */

int
InformixTclDosqlCmd (dummy, interp, objc, objv)
    ClientData      dummy;          /* Not used. */
    Tcl_Interp      *interp;        /* Current interpreter. */
    int             objc;           /* Number of arguments. */
    Tcl_Obj         *CONST objv[];  /* Argument objects. */
{
    char            *logonName;     /* Name of Logon Handle */
    LogonInfo       *logonVars;     /* Logon Handle */
    int             dummyLength;
    int             rc;
    char            *cmd;           /* SQL Statement */

    /*
        Check, if there are at least the Logon Handle and a SQL statement.
    */

    if (objc != 3) {
        AppendResult (interp, "wrong # args: should be \"");
        AppendResult (interp, Tcl_GetStringFromObj ( objv[0], &dummyLength));
        AppendResult (interp, " Logon-Handle SQL-Statement\"");
        return TCL_ERROR;
    }

    logonName = Tcl_GetStringFromObj ( objv[1], &dummyLength);
    if (InformixTclLogonHandle (interp, logonName, &logonVars) 
        == TCL_ERROR) {
        return TCL_ERROR;
    }
    /*
        Get SQL Statement 
    */
    cmd = Tcl_GetStringFromObj (objv[2], &dummyLength);

    /*
        Call the procedure , which handles the SQL request.
    */

    rc=InformixApiDosql (interp, logonVars, cmd);
    return(rc);
}
